@@ -6,9 +6,10 @@ NBDev <- function(counts, design, log.offset, nb.disp, print.progress = TRUE, bi
         log.offset <- rep(0, ncol(counts))
     est.offset <- exp(log.offset)
     
-    deviance.vector <- rep(NA, nrow(counts))
-    means <- matrix(NA, nrow(counts), ncol(counts))
-    parms <- matrix(NA, nrow(counts), ncol(design))
+    deviance.vector <- rep(NA_real_, nrow(counts))
+    means <- matrix(NA_real_, nrow(counts), ncol(counts))
+    parms <- matrix(NA_real_, nrow(counts), ncol(design))
+	BartEpsilons = deviance.vector # NA's
 	design.df=as.data.frame(design)
 	glm.ctrl=glm.control(epsilon = 1e-08, maxit = 1500L, trace = FALSE)
 	fbrNBglm.ctrl=fbrNBglm.control(coefOnly=TRUE, infoParms=list(j=1,k=1,m=1), maxit=1500L, tol=1e-8, standardizeX=TRUE)
@@ -17,16 +18,18 @@ NBDev <- function(counts, design, log.offset, nb.disp, print.progress = TRUE, bi
 	log10=log(10)
 	
     ### For each gene and given design matrix, fit glm using provided negative binomial dispersion estimate
+	if(print.progress) progressBar = txtProgressBar(max = NROW(counts),style=3L)
     for (gn in 1:nrow(counts)) {
         ### If wanted, provide running progress update (eventually once every 5000 genes)
-        if (gn %in% c(2, 10, 100, 500, 1000, 2500, 5000 * (1:200)) & print.progress) 
-            print(paste("Analyzing Gene #", gn))
-        
+        #if (gn %in% c(2, 10, 100, 500, 1000, 2500, 5000 * (1:200)) & print.progress)             print(paste("Analyzing Gene #", gn))
+        if(print.progress) setTxtProgressBar(progressBar, value=gn)
+		
         #### For 2000 Fly genes, glm takes roughly 9 seconds (optim took roughly 21 seconds)
 		glm.fitted <- glmsolve(formula = counts[gn, ] ~ . - 1 + offset(log.offset), family = update.fbrNBfamily(nbLogFamily, overdisp=nb.disp[gn]), data = design.df, control = glm.ctrl, x=TRUE) 
 			
         parms[gn, ] <- glm.fitted$coefficients
-        
+        BartEpsilons[gn] = bartlettFactor(glm.fitted)
+		
         ### Save deviance (used to compute LRT for comparing models and also deviance dispersion estimator)
         deviance.vector[gn] <- glm.fitted$deviance
         
@@ -50,7 +53,8 @@ NBDev <- function(counts, design, log.offset, nb.disp, print.progress = TRUE, bi
 		### NOTE: This line was originally before the Firth bias correction block. 
 		means[gn, ] <- as.vector(exp(design[,tmp.nonNA,drop=FALSE] %*% parms[gn, tmp.nonNA]) * est.offset)
     }
-    return(list(dev = deviance.vector, means = means, parms = parms))
+	if(print.progress) close(progressBar)
+    return(list(dev = deviance.vector, means = means, parms = parms, bartlett.epsilon = BartEpsilons))
 } 
 
 
